# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0.

from awscrt.http import HttpClientConnection, HttpRequest, HttpHeaders
from awscrt.io import ClientBootstrap, ClientTlsContext, is_alpn_available, SocketOptions, TlsConnectionOptions
import awsiot
from concurrent.futures import Future
import json

class DiscoveryClient(object):
    __slots__ = ['_bootstrap', '_tls_context', '_socket_options', '_region', '_tls_connection_options', '_gg_server_name', 'gg_url', 'port']

    def __init__(self, bootstrap, socket_options, tls_context, region):
        assert isinstance(bootstrap, ClientBootstrap)
        assert isinstance(socket_options, SocketOptions)
        assert isinstance(tls_context, ClientTlsContext)
        assert isinstance(region, str)

        self._bootstrap = bootstrap
        self._socket_options = socket_options
        self._region = region
        self._gg_server_name = 'greengrass-ats.iot.{}.amazonaws.com'.format(region)
        self._tls_connection_options = tls_context.new_connection_options()
        self._tls_connection_options.set_server_name(self._gg_server_name)
        self.port = 8443

        if is_alpn_available():
            self._tls_connection_options.set_alpn_list(['x-amzn-http-ca'])
            self.port = 443

    def discover(self, thing_name):

        discovery = dict(
            future=Future(),
            response_body=bytearray())

        def on_incoming_body(http_stream, chunk, **kwargs):
            discovery['response_body'].extend(chunk)

        def on_request_complete(completion_future):
            try:
                response_code = completion_future.result()
                if response_code == 200:
                    payload_str = discovery['response_body'].decode('utf-8')
                    discover_res = DiscoverResponse.from_payload(json.loads(payload_str))
                    discovery['future'].set_result(discover_res)
                else:
                    discovery['future'].set_exception(DiscoveryException('Error during discover call: response_code={}'.format(response_code), response_code))

            except Exception as e:
                discovery['future'].set_exception(e)

        def on_connection_completed(conn_future):
            try:
                connection = conn_future.result()
                headers = HttpHeaders()
                headers.add('host', self._gg_server_name)
                request = HttpRequest(
                    method='GET',
                    path='/greengrass/discover/thing/{}'.format(thing_name),
                    headers=headers)

                http_stream = connection.request(
                    request=request,
                    on_body=on_incoming_body)
               
                http_stream.activate()
                http_stream.completion_future.add_done_callback(on_request_complete)

            except Exception as e:
                discovery['future'].set_exception(e)

        connect_future = HttpClientConnection.new(
            host_name=self._gg_server_name,
            port=self.port,
            socket_options=self._socket_options,
            tls_connection_options = self._tls_connection_options,
            bootstrap = self._bootstrap)

        connect_future.add_done_callback(on_connection_completed)

        return discovery['future']

class DiscoveryException(Exception):
    _slots_ = ['http_response_code', 'message']

    def __init__(self, message, response_code):
        self.http_response_code = response_code
        self.message = message


class ConnectivityInfo(awsiot.ModeledClass):
    __slots__ = ['id', 'host_address', 'metadata', 'port']

    def ___init___(self):
        for slot in self.__slots__:
            setattr(self, slot, None)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> ConnectivityInfo
        new = cls()
        val = payload.get('Id')
        if val is not None:
            new.id = val
        val = payload.get('HostAddress')
        if val is not None:
            new.host_address = val
        val = payload.get('PortNumber')
        if val is not None:
            new.port = val
        val = payload.get('Metadata')
        if val is not None:
            new.metadata = val
        return new

class GGCore(awsiot.ModeledClass):
    __slots__ = ['thing_arn', 'connectivity']

    def ___init___(self):
        for slot in self.__slots__:
            setattr(self, slot, None)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> GGCore
        new = cls()
        val = payload.get('thingArn')
        if val is not None:
            new.thing_arn = val
        val = payload.get('Connectivity')
        if val is not None:
            new.connectivity = [ConnectivityInfo.from_payload(i) for i in val]

        return new

class GGGroup(awsiot.ModeledClass):
    __slots__ = ['gg_group_id', 'cores', 'certificate_authorities']

    def ___init___(self):
       for slot in self.__slots__:
           setattr(self, slot, None)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> GGGroup
        new = cls()
        val = payload.get('GGGroupId')
        if val is not None:
            new.gg_group_id = val
        val = payload.get('Cores')
        if val is not None:
            new.cores = [GGCore.from_payload(i) for i in val]
        val = payload.get('CAs')
        if val is not None:
            new.certificate_authorities = val

        return new

class DiscoverResponse(awsiot.ModeledClass):
    __slots__ = ['gg_groups']

    def ___init___(self):
        for slot in self.__slots__:
            setattr(self, slot, None)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> DiscoverResponse
        new = cls()
        val = payload.get('GGGroups')
        if val is not None:
            new.gg_groups = [GGGroup.from_payload(i) for i in val]

        return new



# Copyright Amazon.com, Inc. or its affiliates.  All rights reserved.
# SPDX-License-Identifier: Apache-2.0.

# This file is generated

import awsiot
import concurrent.futures
import datetime
import typing

class IotShadowClient(awsiot.MqttServiceClient):

    def publish_delete_named_shadow(self, request, qos):
        # type: (DeleteNamedShadowRequest, int) -> concurrent.futures.Future
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#delete-pub-sub-topic

        Parameters:
        request - `DeleteNamedShadowRequest` instance.
        qos     - The Quality of Service guarantee of this message

        Returns a concurrent.futures.Future, whose result will be None if the
        request is successfully published. The Future's result will be an
        exception if the request cannot be published.
        """
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        return self._publish_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/delete'.format(request),
            qos=qos,
            payload=request.to_payload())

    def publish_delete_shadow(self, request, qos):
        # type: (DeleteShadowRequest, int) -> concurrent.futures.Future
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#delete-pub-sub-topic

        Parameters:
        request - `DeleteShadowRequest` instance.
        qos     - The Quality of Service guarantee of this message

        Returns a concurrent.futures.Future, whose result will be None if the
        request is successfully published. The Future's result will be an
        exception if the request cannot be published.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        return self._publish_operation(
            topic='$aws/things/{0.thing_name}/shadow/delete'.format(request),
            qos=qos,
            payload=request.to_payload())

    def publish_get_named_shadow(self, request, qos):
        # type: (GetNamedShadowRequest, int) -> concurrent.futures.Future
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#get-pub-sub-topic

        Parameters:
        request - `GetNamedShadowRequest` instance.
        qos     - The Quality of Service guarantee of this message

        Returns a concurrent.futures.Future, whose result will be None if the
        request is successfully published. The Future's result will be an
        exception if the request cannot be published.
        """
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        return self._publish_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/get'.format(request),
            qos=qos,
            payload=request.to_payload())

    def publish_get_shadow(self, request, qos):
        # type: (GetShadowRequest, int) -> concurrent.futures.Future
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#get-pub-sub-topic

        Parameters:
        request - `GetShadowRequest` instance.
        qos     - The Quality of Service guarantee of this message

        Returns a concurrent.futures.Future, whose result will be None if the
        request is successfully published. The Future's result will be an
        exception if the request cannot be published.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        return self._publish_operation(
            topic='$aws/things/{0.thing_name}/shadow/get'.format(request),
            qos=qos,
            payload=request.to_payload())

    def publish_update_named_shadow(self, request, qos):
        # type: (UpdateNamedShadowRequest, int) -> concurrent.futures.Future
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#update-pub-sub-topic

        Parameters:
        request - `UpdateNamedShadowRequest` instance.
        qos     - The Quality of Service guarantee of this message

        Returns a concurrent.futures.Future, whose result will be None if the
        request is successfully published. The Future's result will be an
        exception if the request cannot be published.
        """
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        return self._publish_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/update'.format(request),
            qos=qos,
            payload=request.to_payload())

    def publish_update_shadow(self, request, qos):
        # type: (UpdateShadowRequest, int) -> concurrent.futures.Future
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#update-pub-sub-topic

        Parameters:
        request - `UpdateShadowRequest` instance.
        qos     - The Quality of Service guarantee of this message

        Returns a concurrent.futures.Future, whose result will be None if the
        request is successfully published. The Future's result will be an
        exception if the request cannot be published.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        return self._publish_operation(
            topic='$aws/things/{0.thing_name}/shadow/update'.format(request),
            qos=qos,
            payload=request.to_payload())

    def subscribe_to_delete_named_shadow_accepted(self, request, qos, callback):
        # type: (DeleteNamedShadowSubscriptionRequest, int, typing.Callable[[DeleteShadowResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#delete-accepted-pub-sub-topic

        Parameters:
        request - `DeleteNamedShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `DeleteShadowResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/delete/accepted'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=DeleteShadowResponse.from_payload)

    def subscribe_to_delete_named_shadow_rejected(self, request, qos, callback):
        # type: (DeleteNamedShadowSubscriptionRequest, int, typing.Callable[[ErrorResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#delete-rejected-pub-sub-topic

        Parameters:
        request - `DeleteNamedShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `ErrorResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/delete/rejected'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=ErrorResponse.from_payload)

    def subscribe_to_delete_shadow_accepted(self, request, qos, callback):
        # type: (DeleteShadowSubscriptionRequest, int, typing.Callable[[DeleteShadowResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#delete-accepted-pub-sub-topic

        Parameters:
        request - `DeleteShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `DeleteShadowResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/delete/accepted'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=DeleteShadowResponse.from_payload)

    def subscribe_to_delete_shadow_rejected(self, request, qos, callback):
        # type: (DeleteShadowSubscriptionRequest, int, typing.Callable[[ErrorResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#delete-rejected-pub-sub-topic

        Parameters:
        request - `DeleteShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `ErrorResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/delete/rejected'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=ErrorResponse.from_payload)

    def subscribe_to_get_named_shadow_accepted(self, request, qos, callback):
        # type: (GetNamedShadowSubscriptionRequest, int, typing.Callable[[GetShadowResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#get-accepted-pub-sub-topic

        Parameters:
        request - `GetNamedShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `GetShadowResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/get/accepted'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=GetShadowResponse.from_payload)

    def subscribe_to_get_named_shadow_rejected(self, request, qos, callback):
        # type: (GetNamedShadowSubscriptionRequest, int, typing.Callable[[ErrorResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#get-rejected-pub-sub-topic

        Parameters:
        request - `GetNamedShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `ErrorResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/get/rejected'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=ErrorResponse.from_payload)

    def subscribe_to_get_shadow_accepted(self, request, qos, callback):
        # type: (GetShadowSubscriptionRequest, int, typing.Callable[[GetShadowResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#get-accepted-pub-sub-topic

        Parameters:
        request - `GetShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `GetShadowResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/get/accepted'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=GetShadowResponse.from_payload)

    def subscribe_to_get_shadow_rejected(self, request, qos, callback):
        # type: (GetShadowSubscriptionRequest, int, typing.Callable[[ErrorResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#get-rejected-pub-sub-topic

        Parameters:
        request - `GetShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `ErrorResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/get/rejected'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=ErrorResponse.from_payload)

    def subscribe_to_named_shadow_delta_updated_events(self, request, qos, callback):
        # type: (NamedShadowDeltaUpdatedSubscriptionRequest, int, typing.Callable[[ShadowDeltaUpdatedEvent], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#update-delta-pub-sub-topic

        Parameters:
        request - `NamedShadowDeltaUpdatedSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `ShadowDeltaUpdatedEvent`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/update/delta'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=ShadowDeltaUpdatedEvent.from_payload)

    def subscribe_to_named_shadow_updated_events(self, request, qos, callback):
        # type: (NamedShadowUpdatedSubscriptionRequest, int, typing.Callable[[ShadowUpdatedEvent], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#update-documents-pub-sub-topic

        Parameters:
        request - `NamedShadowUpdatedSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `ShadowUpdatedEvent`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/update/documents'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=ShadowUpdatedEvent.from_payload)

    def subscribe_to_shadow_delta_updated_events(self, request, qos, callback):
        # type: (ShadowDeltaUpdatedSubscriptionRequest, int, typing.Callable[[ShadowDeltaUpdatedEvent], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#update-delta-pub-sub-topic

        Parameters:
        request - `ShadowDeltaUpdatedSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `ShadowDeltaUpdatedEvent`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/update/delta'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=ShadowDeltaUpdatedEvent.from_payload)

    def subscribe_to_shadow_updated_events(self, request, qos, callback):
        # type: (ShadowUpdatedSubscriptionRequest, int, typing.Callable[[ShadowUpdatedEvent], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#update-documents-pub-sub-topic

        Parameters:
        request - `ShadowUpdatedSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `ShadowUpdatedEvent`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/update/documents'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=ShadowUpdatedEvent.from_payload)

    def subscribe_to_update_named_shadow_accepted(self, request, qos, callback):
        # type: (UpdateNamedShadowSubscriptionRequest, int, typing.Callable[[UpdateShadowResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#update-accepted-pub-sub-topic

        Parameters:
        request - `UpdateNamedShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `UpdateShadowResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/update/accepted'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=UpdateShadowResponse.from_payload)

    def subscribe_to_update_named_shadow_rejected(self, request, qos, callback):
        # type: (UpdateNamedShadowSubscriptionRequest, int, typing.Callable[[ErrorResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#update-rejected-pub-sub-topic

        Parameters:
        request - `UpdateNamedShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `ErrorResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.shadow_name:
            raise ValueError("request.shadow_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/name/{0.shadow_name}/update/rejected'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=ErrorResponse.from_payload)

    def subscribe_to_update_shadow_accepted(self, request, qos, callback):
        # type: (UpdateShadowSubscriptionRequest, int, typing.Callable[[UpdateShadowResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#update-accepted-pub-sub-topic

        Parameters:
        request - `UpdateShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `UpdateShadowResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/update/accepted'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=UpdateShadowResponse.from_payload)

    def subscribe_to_update_shadow_rejected(self, request, qos, callback):
        # type: (UpdateShadowSubscriptionRequest, int, typing.Callable[[ErrorResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/device-shadow-mqtt.html#update-rejected-pub-sub-topic

        Parameters:
        request - `UpdateShadowSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `ErrorResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/shadow/update/rejected'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=ErrorResponse.from_payload)

class DeleteNamedShadowRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *shadow_name* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'shadow_name', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a DeleteNamedShadowRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *shadow_name* (``str``)
            * *thing_name* (``str``)
        """

        self.client_token = kwargs.get('client_token')
        self.shadow_name = kwargs.get('shadow_name')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'shadow_name', 'thing_name'], args):
            setattr(self, key, val)

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.client_token is not None:
            payload['clientToken'] = self.client_token
        return payload

class DeleteNamedShadowSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *shadow_name* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['shadow_name', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a DeleteNamedShadowSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *shadow_name* (``str``)
            * *thing_name* (``str``)
        """

        self.shadow_name = kwargs.get('shadow_name')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['shadow_name', 'thing_name'], args):
            setattr(self, key, val)

class DeleteShadowRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a DeleteShadowRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *thing_name* (``str``)
        """

        self.client_token = kwargs.get('client_token')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        if len(args) == 1:
            # if one positional parameter was passed, it must be the thing_name because it is
            # a required parameter
            setattr(self, 'thing_name', args[0])
        elif len(args) == 2:
            # if two parameters were passed, it must be the case that they are in the order
            # of client_token, thing_name; nothing else could have ever worked
            setattr(self, 'client_token', args[0])
            setattr(self, 'thing_name', args[1])

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.client_token is not None:
            payload['clientToken'] = self.client_token
        return payload

class DeleteShadowResponse(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *timestamp* (``datetime.datetime``)
        * *version* (``int``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'timestamp', 'version']

    def __init__(self, *args, **kwargs):
        r"""Initializes a DeleteShadowResponse instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *timestamp* (``datetime.datetime``)
            * *version* (``int``)
        """

        self.client_token = kwargs.get('client_token')
        self.timestamp = kwargs.get('timestamp')
        self.version = kwargs.get('version')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['timestamp', 'version'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> DeleteShadowResponse
        new = cls()
        val = payload.get('clientToken')
        if val is not None:
            new.client_token = val
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        val = payload.get('version')
        if val is not None:
            new.version = val
        return new

class DeleteShadowSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a DeleteShadowSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *thing_name* (``str``)
        """

        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['thing_name'], args):
            setattr(self, key, val)

class ErrorResponse(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *code* (``int``)
        * *message* (``str``)
        * *timestamp* (``datetime.datetime``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'code', 'message', 'timestamp']

    def __init__(self, *args, **kwargs):
        r"""Initializes a ErrorResponse instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *code* (``int``)
            * *message* (``str``)
            * *timestamp* (``datetime.datetime``)
        """

        self.client_token = kwargs.get('client_token')
        self.code = kwargs.get('code')
        self.message = kwargs.get('message')
        self.timestamp = kwargs.get('timestamp')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'code', 'message', 'timestamp'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> ErrorResponse
        new = cls()
        val = payload.get('clientToken')
        if val is not None:
            new.client_token = val
        val = payload.get('code')
        if val is not None:
            new.code = val
        val = payload.get('message')
        if val is not None:
            new.message = val
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        return new

class GetNamedShadowRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *shadow_name* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'shadow_name', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a GetNamedShadowRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *shadow_name* (``str``)
            * *thing_name* (``str``)
        """

        self.client_token = kwargs.get('client_token')
        self.shadow_name = kwargs.get('shadow_name')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'shadow_name', 'thing_name'], args):
            setattr(self, key, val)

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.client_token is not None:
            payload['clientToken'] = self.client_token
        return payload

class GetNamedShadowSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *shadow_name* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['shadow_name', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a GetNamedShadowSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *shadow_name* (``str``)
            * *thing_name* (``str``)
        """

        self.shadow_name = kwargs.get('shadow_name')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['shadow_name', 'thing_name'], args):
            setattr(self, key, val)

class GetShadowRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a GetShadowRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *thing_name* (``str``)
        """

        self.client_token = kwargs.get('client_token')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        if len(args) == 1:
            # if one positional parameter was passed, it must be the thing_name because it is
            # a required parameter
            setattr(self, 'thing_name', args[0])
        elif len(args) == 2:
            # if two parameters were passed, it must be the case that they are in the order
            # of client_token, thing_name; nothing else could have ever worked
            setattr(self, 'client_token', args[0])
            setattr(self, 'thing_name', args[1])

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.client_token is not None:
            payload['clientToken'] = self.client_token
        return payload

class GetShadowResponse(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *metadata* (``ShadowMetadata``)
        * *state* (``ShadowStateWithDelta``)
        * *timestamp* (``datetime.datetime``)
        * *version* (``int``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'metadata', 'state', 'timestamp', 'version']

    def __init__(self, *args, **kwargs):
        r"""Initializes a GetShadowResponse instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *metadata* (``ShadowMetadata``)
            * *state* (``ShadowStateWithDelta``)
            * *timestamp* (``datetime.datetime``)
            * *version* (``int``)
        """

        self.client_token = kwargs.get('client_token')
        self.metadata = kwargs.get('metadata')
        self.state = kwargs.get('state')
        self.timestamp = kwargs.get('timestamp')
        self.version = kwargs.get('version')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['metadata', 'state', 'timestamp', 'version'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> GetShadowResponse
        new = cls()
        val = payload.get('clientToken')
        if val is not None:
            new.client_token = val
        val = payload.get('metadata')
        if val is not None:
            new.metadata = ShadowMetadata.from_payload(val)
        val = payload.get('state')
        if val is not None:
            new.state = ShadowStateWithDelta.from_payload(val)
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        val = payload.get('version')
        if val is not None:
            new.version = val
        return new

class GetShadowSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a GetShadowSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *thing_name* (``str``)
        """

        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['thing_name'], args):
            setattr(self, key, val)

class NamedShadowDeltaUpdatedSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *shadow_name* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['shadow_name', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a NamedShadowDeltaUpdatedSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *shadow_name* (``str``)
            * *thing_name* (``str``)
        """

        self.shadow_name = kwargs.get('shadow_name')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['shadow_name', 'thing_name'], args):
            setattr(self, key, val)

class NamedShadowUpdatedSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *shadow_name* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['shadow_name', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a NamedShadowUpdatedSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *shadow_name* (``str``)
            * *thing_name* (``str``)
        """

        self.shadow_name = kwargs.get('shadow_name')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['shadow_name', 'thing_name'], args):
            setattr(self, key, val)

class ShadowDeltaUpdatedEvent(awsiot.ModeledClass):
    r"""
    Attributes:
        * *metadata* (``typing.Dict[str, typing.Any]``)
        * *state* (``typing.Dict[str, typing.Any]``)
        * *timestamp* (``datetime.datetime``)
        * *version* (``int``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['metadata', 'state', 'timestamp', 'version']

    def __init__(self, *args, **kwargs):
        r"""Initializes a ShadowDeltaUpdatedEvent instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *metadata* (``typing.Dict[str, typing.Any]``)
            * *state* (``typing.Dict[str, typing.Any]``)
            * *timestamp* (``datetime.datetime``)
            * *version* (``int``)
        """

        self.metadata = kwargs.get('metadata')
        self.state = kwargs.get('state')
        self.timestamp = kwargs.get('timestamp')
        self.version = kwargs.get('version')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['metadata', 'state', 'timestamp', 'version'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> ShadowDeltaUpdatedEvent
        new = cls()
        val = payload.get('metadata')
        if val is not None:
            new.metadata = val
        val = payload.get('state')
        if val is not None:
            new.state = val
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        val = payload.get('version')
        if val is not None:
            new.version = val
        return new

class ShadowDeltaUpdatedSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a ShadowDeltaUpdatedSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *thing_name* (``str``)
        """

        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['thing_name'], args):
            setattr(self, key, val)

class ShadowMetadata(awsiot.ModeledClass):
    r"""
    Attributes:
        * *desired* (``typing.Dict[str, typing.Any]``)
        * *reported* (``typing.Dict[str, typing.Any]``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['desired', 'reported']

    def __init__(self, *args, **kwargs):
        r"""Initializes a ShadowMetadata instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *desired* (``typing.Dict[str, typing.Any]``)
            * *reported* (``typing.Dict[str, typing.Any]``)
        """

        self.desired = kwargs.get('desired')
        self.reported = kwargs.get('reported')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['desired', 'reported'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> ShadowMetadata
        new = cls()
        val = payload.get('desired')
        if val is not None:
            new.desired = val
        val = payload.get('reported')
        if val is not None:
            new.reported = val
        return new

class ShadowState(awsiot.ModeledClass):
    r"""
    Attributes:
        * *desired* (``typing.Dict[str, typing.Any]``)
        * *reported* (``typing.Dict[str, typing.Any]``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['desired', 'reported']

    def __init__(self, *args, **kwargs):
        r"""Initializes a ShadowState instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *desired* (``typing.Dict[str, typing.Any]``)
            * *reported* (``typing.Dict[str, typing.Any]``)
        """

        self.desired = kwargs.get('desired')
        self.reported = kwargs.get('reported')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['desired', 'reported'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> ShadowState
        new = cls()
        val = payload.get('desired')
        if val is not None:
            new.desired = val
        val = payload.get('reported')
        if val is not None:
            new.reported = val
        return new

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.desired is not None:
            payload['desired'] = self.desired
        if self.reported is not None:
            payload['reported'] = self.reported
        return payload

class ShadowStateWithDelta(awsiot.ModeledClass):
    r"""
    Attributes:
        * *delta* (``typing.Dict[str, typing.Any]``)
        * *desired* (``typing.Dict[str, typing.Any]``)
        * *reported* (``typing.Dict[str, typing.Any]``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['delta', 'desired', 'reported']

    def __init__(self, *args, **kwargs):
        r"""Initializes a ShadowStateWithDelta instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *delta* (``typing.Dict[str, typing.Any]``)
            * *desired* (``typing.Dict[str, typing.Any]``)
            * *reported* (``typing.Dict[str, typing.Any]``)
        """

        self.delta = kwargs.get('delta')
        self.desired = kwargs.get('desired')
        self.reported = kwargs.get('reported')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['delta', 'desired', 'reported'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> ShadowStateWithDelta
        new = cls()
        val = payload.get('delta')
        if val is not None:
            new.delta = val
        val = payload.get('desired')
        if val is not None:
            new.desired = val
        val = payload.get('reported')
        if val is not None:
            new.reported = val
        return new

class ShadowUpdatedEvent(awsiot.ModeledClass):
    r"""
    Attributes:
        * *current* (``ShadowUpdatedSnapshot``)
        * *previous* (``ShadowUpdatedSnapshot``)
        * *timestamp* (``datetime.datetime``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['current', 'previous', 'timestamp']

    def __init__(self, *args, **kwargs):
        r"""Initializes a ShadowUpdatedEvent instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *current* (``ShadowUpdatedSnapshot``)
            * *previous* (``ShadowUpdatedSnapshot``)
            * *timestamp* (``datetime.datetime``)
        """

        self.current = kwargs.get('current')
        self.previous = kwargs.get('previous')
        self.timestamp = kwargs.get('timestamp')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['current', 'previous', 'timestamp'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> ShadowUpdatedEvent
        new = cls()
        val = payload.get('current')
        if val is not None:
            new.current = ShadowUpdatedSnapshot.from_payload(val)
        val = payload.get('previous')
        if val is not None:
            new.previous = ShadowUpdatedSnapshot.from_payload(val)
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        return new

class ShadowUpdatedSnapshot(awsiot.ModeledClass):
    r"""
    Attributes:
        * *metadata* (``ShadowMetadata``)
        * *state* (``ShadowState``)
        * *version* (``int``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['metadata', 'state', 'version']

    def __init__(self, *args, **kwargs):
        r"""Initializes a ShadowUpdatedSnapshot instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *metadata* (``ShadowMetadata``)
            * *state* (``ShadowState``)
            * *version* (``int``)
        """

        self.metadata = kwargs.get('metadata')
        self.state = kwargs.get('state')
        self.version = kwargs.get('version')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['metadata', 'state', 'version'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> ShadowUpdatedSnapshot
        new = cls()
        val = payload.get('metadata')
        if val is not None:
            new.metadata = ShadowMetadata.from_payload(val)
        val = payload.get('state')
        if val is not None:
            new.state = ShadowState.from_payload(val)
        val = payload.get('version')
        if val is not None:
            new.version = val
        return new

class ShadowUpdatedSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a ShadowUpdatedSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *thing_name* (``str``)
        """

        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['thing_name'], args):
            setattr(self, key, val)

class UpdateNamedShadowRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *shadow_name* (``str``)
        * *state* (``ShadowState``)
        * *thing_name* (``str``)
        * *version* (``int``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'shadow_name', 'state', 'thing_name', 'version']

    def __init__(self, *args, **kwargs):
        r"""Initializes a UpdateNamedShadowRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *shadow_name* (``str``)
            * *state* (``ShadowState``)
            * *thing_name* (``str``)
            * *version* (``int``)
        """

        self.client_token = kwargs.get('client_token')
        self.shadow_name = kwargs.get('shadow_name')
        self.state = kwargs.get('state')
        self.thing_name = kwargs.get('thing_name')
        self.version = kwargs.get('version')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'shadow_name', 'state', 'thing_name', 'version'], args):
            setattr(self, key, val)

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.client_token is not None:
            payload['clientToken'] = self.client_token
        if self.state is not None:
            payload['state'] = self.state.to_payload()
        if self.version is not None:
            payload['version'] = self.version
        return payload

class UpdateNamedShadowSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *shadow_name* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['shadow_name', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a UpdateNamedShadowSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *shadow_name* (``str``)
            * *thing_name* (``str``)
        """

        self.shadow_name = kwargs.get('shadow_name')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['shadow_name', 'thing_name'], args):
            setattr(self, key, val)

class UpdateShadowRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *state* (``ShadowState``)
        * *thing_name* (``str``)
        * *version* (``int``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'state', 'thing_name', 'version']

    def __init__(self, *args, **kwargs):
        r"""Initializes a UpdateShadowRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *state* (``ShadowState``)
            * *thing_name* (``str``)
            * *version* (``int``)
        """

        self.client_token = kwargs.get('client_token')
        self.state = kwargs.get('state')
        self.thing_name = kwargs.get('thing_name')
        self.version = kwargs.get('version')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'state', 'thing_name', 'version'], args):
            setattr(self, key, val)

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.client_token is not None:
            payload['clientToken'] = self.client_token
        if self.state is not None:
            payload['state'] = self.state.to_payload()
        if self.version is not None:
            payload['version'] = self.version
        return payload

class UpdateShadowResponse(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *metadata* (``ShadowMetadata``)
        * *state* (``ShadowState``)
        * *timestamp* (``datetime.datetime``)
        * *version* (``int``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'metadata', 'state', 'timestamp', 'version']

    def __init__(self, *args, **kwargs):
        r"""Initializes a UpdateShadowResponse instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *metadata* (``ShadowMetadata``)
            * *state* (``ShadowState``)
            * *timestamp* (``datetime.datetime``)
            * *version* (``int``)
        """

        self.client_token = kwargs.get('client_token')
        self.metadata = kwargs.get('metadata')
        self.state = kwargs.get('state')
        self.timestamp = kwargs.get('timestamp')
        self.version = kwargs.get('version')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'metadata', 'state', 'timestamp', 'version'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> UpdateShadowResponse
        new = cls()
        val = payload.get('clientToken')
        if val is not None:
            new.client_token = val
        val = payload.get('metadata')
        if val is not None:
            new.metadata = ShadowMetadata.from_payload(val)
        val = payload.get('state')
        if val is not None:
            new.state = ShadowState.from_payload(val)
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        val = payload.get('version')
        if val is not None:
            new.version = val
        return new

class UpdateShadowSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a UpdateShadowSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *thing_name* (``str``)
        """

        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['thing_name'], args):
            setattr(self, key, val)




#!/usr/bin/env python

# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0.

from setuptools import setup

setup(
    name='awsiotsdk',
    version='1.3.1',
    description='AWS IoT SDK based on the AWS Common Runtime',
    author='AWS SDK Common Runtime Team',
    url='https://github.com/aws/aws-iot-device-sdk-python-v2',
    packages = ['awsiot'],
    install_requires=[
        'awscrt==0.6.1',
        'futures;python_version<"3.2"',
        'typing;python_version<"3.5"',
    ],
    python_requires='>=2.7',
)



# Copyright Amazon.com, Inc. or its affiliates.  All rights reserved.
# SPDX-License-Identifier: Apache-2.0.

# This file is generated

import awsiot
import concurrent.futures
import datetime
import typing

class IotJobsClient(awsiot.MqttServiceClient):

    def publish_describe_job_execution(self, request, qos):
        # type: (DescribeJobExecutionRequest, int) -> concurrent.futures.Future
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-describejobexecution

        Parameters:
        request - `DescribeJobExecutionRequest` instance.
        qos     - The Quality of Service guarantee of this message

        Returns a concurrent.futures.Future, whose result will be None if the
        request is successfully published. The Future's result will be an
        exception if the request cannot be published.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.job_id:
            raise ValueError("request.job_id is required")

        return self._publish_operation(
            topic='$aws/things/{0.thing_name}/jobs/{0.job_id}/get'.format(request),
            qos=qos,
            payload=request.to_payload())

    def publish_get_pending_job_executions(self, request, qos):
        # type: (GetPendingJobExecutionsRequest, int) -> concurrent.futures.Future
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-getpendingjobexecutions

        Parameters:
        request - `GetPendingJobExecutionsRequest` instance.
        qos     - The Quality of Service guarantee of this message

        Returns a concurrent.futures.Future, whose result will be None if the
        request is successfully published. The Future's result will be an
        exception if the request cannot be published.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        return self._publish_operation(
            topic='$aws/things/{0.thing_name}/jobs/get'.format(request),
            qos=qos,
            payload=request.to_payload())

    def publish_start_next_pending_job_execution(self, request, qos):
        # type: (StartNextPendingJobExecutionRequest, int) -> concurrent.futures.Future
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-startnextpendingjobexecution

        Parameters:
        request - `StartNextPendingJobExecutionRequest` instance.
        qos     - The Quality of Service guarantee of this message

        Returns a concurrent.futures.Future, whose result will be None if the
        request is successfully published. The Future's result will be an
        exception if the request cannot be published.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        return self._publish_operation(
            topic='$aws/things/{0.thing_name}/jobs/start-next'.format(request),
            qos=qos,
            payload=request.to_payload())

    def publish_update_job_execution(self, request, qos):
        # type: (UpdateJobExecutionRequest, int) -> concurrent.futures.Future
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-updatejobexecution

        Parameters:
        request - `UpdateJobExecutionRequest` instance.
        qos     - The Quality of Service guarantee of this message

        Returns a concurrent.futures.Future, whose result will be None if the
        request is successfully published. The Future's result will be an
        exception if the request cannot be published.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.job_id:
            raise ValueError("request.job_id is required")

        return self._publish_operation(
            topic='$aws/things/{0.thing_name}/jobs/{0.job_id}/update'.format(request),
            qos=qos,
            payload=request.to_payload())

    def subscribe_to_describe_job_execution_accepted(self, request, qos, callback):
        # type: (DescribeJobExecutionSubscriptionRequest, int, typing.Callable[[DescribeJobExecutionResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-describejobexecution

        Parameters:
        request - `DescribeJobExecutionSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `DescribeJobExecutionResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.job_id:
            raise ValueError("request.job_id is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/jobs/{0.job_id}/get/accepted'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=DescribeJobExecutionResponse.from_payload)

    def subscribe_to_describe_job_execution_rejected(self, request, qos, callback):
        # type: (DescribeJobExecutionSubscriptionRequest, int, typing.Callable[[RejectedError], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-describejobexecution

        Parameters:
        request - `DescribeJobExecutionSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `RejectedError`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")
        if not request.job_id:
            raise ValueError("request.job_id is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/jobs/{0.job_id}/get/rejected'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=RejectedError.from_payload)

    def subscribe_to_get_pending_job_executions_accepted(self, request, qos, callback):
        # type: (GetPendingJobExecutionsSubscriptionRequest, int, typing.Callable[[GetPendingJobExecutionsResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-getpendingjobexecutions

        Parameters:
        request - `GetPendingJobExecutionsSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `GetPendingJobExecutionsResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/jobs/get/accepted'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=GetPendingJobExecutionsResponse.from_payload)

    def subscribe_to_get_pending_job_executions_rejected(self, request, qos, callback):
        # type: (GetPendingJobExecutionsSubscriptionRequest, int, typing.Callable[[RejectedError], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-getpendingjobexecutions

        Parameters:
        request - `GetPendingJobExecutionsSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `RejectedError`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/jobs/get/rejected'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=RejectedError.from_payload)

    def subscribe_to_job_executions_changed_events(self, request, qos, callback):
        # type: (JobExecutionsChangedSubscriptionRequest, int, typing.Callable[[JobExecutionsChangedEvent], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-jobexecutionschanged

        Parameters:
        request - `JobExecutionsChangedSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `JobExecutionsChangedEvent`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/jobs/notify'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=JobExecutionsChangedEvent.from_payload)

    def subscribe_to_next_job_execution_changed_events(self, request, qos, callback):
        # type: (NextJobExecutionChangedSubscriptionRequest, int, typing.Callable[[NextJobExecutionChangedEvent], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-nextjobexecutionchanged

        Parameters:
        request - `NextJobExecutionChangedSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `NextJobExecutionChangedEvent`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/jobs/notify-next'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=NextJobExecutionChangedEvent.from_payload)

    def subscribe_to_start_next_pending_job_execution_accepted(self, request, qos, callback):
        # type: (StartNextPendingJobExecutionSubscriptionRequest, int, typing.Callable[[StartNextJobExecutionResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-startnextpendingjobexecution

        Parameters:
        request - `StartNextPendingJobExecutionSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `StartNextJobExecutionResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/jobs/start-next/accepted'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=StartNextJobExecutionResponse.from_payload)

    def subscribe_to_start_next_pending_job_execution_rejected(self, request, qos, callback):
        # type: (StartNextPendingJobExecutionSubscriptionRequest, int, typing.Callable[[RejectedError], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-startnextpendingjobexecution

        Parameters:
        request - `StartNextPendingJobExecutionSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `RejectedError`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/jobs/start-next/rejected'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=RejectedError.from_payload)

    def subscribe_to_update_job_execution_accepted(self, request, qos, callback):
        # type: (UpdateJobExecutionSubscriptionRequest, int, typing.Callable[[UpdateJobExecutionResponse], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-updatejobexecution

        Parameters:
        request - `UpdateJobExecutionSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `UpdateJobExecutionResponse`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.job_id:
            raise ValueError("request.job_id is required")
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/jobs/{0.job_id}/update/accepted'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=UpdateJobExecutionResponse.from_payload)

    def subscribe_to_update_job_execution_rejected(self, request, qos, callback):
        # type: (UpdateJobExecutionSubscriptionRequest, int, typing.Callable[[RejectedError], None]) -> typing.Tuple[concurrent.futures.Future, str]
        """
        API Docs: https://docs.aws.amazon.com/iot/latest/developerguide/jobs-api.html#mqtt-updatejobexecution

        Parameters:
        request - `UpdateJobExecutionSubscriptionRequest` instance.
        qos     - The Quality of Service guarantee of this message
        callback - Callback to invoke each time the event is received.
                The callback should take 1 argument of type `RejectedError`.
                The callback is not expected to return anything.

        Returns two values immediately. The first is a `concurrent.futures.Future`
        which will contain a result of `None` when the server has acknowledged
        the subscription, or an exception if the subscription fails. The second
        value is a topic which may be passed to `unsubscribe()` to stop
        receiving messages. Note that messages may arrive before the
        subscription is acknowledged.
        """
        if not request.job_id:
            raise ValueError("request.job_id is required")
        if not request.thing_name:
            raise ValueError("request.thing_name is required")

        if not callable(callback):
            raise ValueError("callback is required")

        return self._subscribe_operation(
            topic='$aws/things/{0.thing_name}/jobs/{0.job_id}/update/rejected'.format(request),
            qos=qos,
            callback=callback,
            payload_to_class_fn=RejectedError.from_payload)

class DescribeJobExecutionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *execution_number* (``int``)
        * *include_job_document* (``bool``)
        * *job_id* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'execution_number', 'include_job_document', 'job_id', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a DescribeJobExecutionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *execution_number* (``int``)
            * *include_job_document* (``bool``)
            * *job_id* (``str``)
            * *thing_name* (``str``)
        """

        self.client_token = kwargs.get('client_token')
        self.execution_number = kwargs.get('execution_number')
        self.include_job_document = kwargs.get('include_job_document')
        self.job_id = kwargs.get('job_id')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'execution_number', 'include_job_document', 'job_id', 'thing_name'], args):
            setattr(self, key, val)

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.client_token is not None:
            payload['clientToken'] = self.client_token
        if self.execution_number is not None:
            payload['executionNumber'] = self.execution_number
        if self.include_job_document is not None:
            payload['includeJobDocument'] = self.include_job_document
        return payload

class DescribeJobExecutionResponse(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *execution* (``JobExecutionData``)
        * *timestamp* (``datetime.datetime``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'execution', 'timestamp']

    def __init__(self, *args, **kwargs):
        r"""Initializes a DescribeJobExecutionResponse instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *execution* (``JobExecutionData``)
            * *timestamp* (``datetime.datetime``)
        """

        self.client_token = kwargs.get('client_token')
        self.execution = kwargs.get('execution')
        self.timestamp = kwargs.get('timestamp')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'execution', 'timestamp'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> DescribeJobExecutionResponse
        new = cls()
        val = payload.get('clientToken')
        if val is not None:
            new.client_token = val
        val = payload.get('execution')
        if val is not None:
            new.execution = JobExecutionData.from_payload(val)
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        return new

class DescribeJobExecutionSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *job_id* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['job_id', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a DescribeJobExecutionSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *job_id* (``str``)
            * *thing_name* (``str``)
        """

        self.job_id = kwargs.get('job_id')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['job_id', 'thing_name'], args):
            setattr(self, key, val)

class GetPendingJobExecutionsRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a GetPendingJobExecutionsRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *thing_name* (``str``)
        """

        self.client_token = kwargs.get('client_token')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'thing_name'], args):
            setattr(self, key, val)

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.client_token is not None:
            payload['clientToken'] = self.client_token
        return payload

class GetPendingJobExecutionsResponse(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *in_progress_jobs* (``typing.List[JobExecutionSummary]``)
        * *queued_jobs* (``typing.List[JobExecutionSummary]``)
        * *timestamp* (``datetime.datetime``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'in_progress_jobs', 'queued_jobs', 'timestamp']

    def __init__(self, *args, **kwargs):
        r"""Initializes a GetPendingJobExecutionsResponse instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *in_progress_jobs* (``typing.List[JobExecutionSummary]``)
            * *queued_jobs* (``typing.List[JobExecutionSummary]``)
            * *timestamp* (``datetime.datetime``)
        """

        self.client_token = kwargs.get('client_token')
        self.in_progress_jobs = kwargs.get('in_progress_jobs')
        self.queued_jobs = kwargs.get('queued_jobs')
        self.timestamp = kwargs.get('timestamp')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'in_progress_jobs', 'queued_jobs', 'timestamp'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> GetPendingJobExecutionsResponse
        new = cls()
        val = payload.get('clientToken')
        if val is not None:
            new.client_token = val
        val = payload.get('inProgressJobs')
        if val is not None:
            new.in_progress_jobs = [JobExecutionSummary.from_payload(i) for i in val]
        val = payload.get('queuedJobs')
        if val is not None:
            new.queued_jobs = [JobExecutionSummary.from_payload(i) for i in val]
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        return new

class GetPendingJobExecutionsSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a GetPendingJobExecutionsSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *thing_name* (``str``)
        """

        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['thing_name'], args):
            setattr(self, key, val)

class JobExecutionData(awsiot.ModeledClass):
    r"""
    Attributes:
        * *execution_number* (``int``)
        * *job_document* (``typing.Dict[str, typing.Any]``)
        * *job_id* (``str``)
        * *last_updated_at* (``datetime.datetime``)
        * *queued_at* (``datetime.datetime``)
        * *started_at* (``datetime.datetime``)
        * *status* (``str``)
        * *status_details* (``typing.Dict[str, str]``)
        * *thing_name* (``str``)
        * *version_number* (``int``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['execution_number', 'job_document', 'job_id', 'last_updated_at', 'queued_at', 'started_at', 'status', 'status_details', 'thing_name', 'version_number']

    def __init__(self, *args, **kwargs):
        r"""Initializes a JobExecutionData instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *execution_number* (``int``)
            * *job_document* (``typing.Dict[str, typing.Any]``)
            * *job_id* (``str``)
            * *last_updated_at* (``datetime.datetime``)
            * *queued_at* (``datetime.datetime``)
            * *started_at* (``datetime.datetime``)
            * *status* (``str``)
            * *status_details* (``typing.Dict[str, str]``)
            * *thing_name* (``str``)
            * *version_number* (``int``)
        """

        self.execution_number = kwargs.get('execution_number')
        self.job_document = kwargs.get('job_document')
        self.job_id = kwargs.get('job_id')
        self.last_updated_at = kwargs.get('last_updated_at')
        self.queued_at = kwargs.get('queued_at')
        self.started_at = kwargs.get('started_at')
        self.status = kwargs.get('status')
        self.status_details = kwargs.get('status_details')
        self.thing_name = kwargs.get('thing_name')
        self.version_number = kwargs.get('version_number')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['execution_number', 'job_document', 'job_id', 'last_updated_at', 'queued_at', 'started_at', 'status', 'status_details', 'thing_name', 'version_number'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> JobExecutionData
        new = cls()
        val = payload.get('executionNumber')
        if val is not None:
            new.execution_number = val
        val = payload.get('jobDocument')
        if val is not None:
            new.job_document = val
        val = payload.get('jobId')
        if val is not None:
            new.job_id = val
        val = payload.get('lastUpdatedAt')
        if val is not None:
            new.last_updated_at = datetime.datetime.fromtimestamp(val)
        val = payload.get('queuedAt')
        if val is not None:
            new.queued_at = datetime.datetime.fromtimestamp(val)
        val = payload.get('startedAt')
        if val is not None:
            new.started_at = datetime.datetime.fromtimestamp(val)
        val = payload.get('status')
        if val is not None:
            new.status = val
        val = payload.get('statusDetails')
        if val is not None:
            new.status_details = val
        val = payload.get('thingName')
        if val is not None:
            new.thing_name = val
        val = payload.get('versionNumber')
        if val is not None:
            new.version_number = val
        return new

class JobExecutionState(awsiot.ModeledClass):
    r"""
    Attributes:
        * *status* (``str``)
        * *status_details* (``typing.Dict[str, str]``)
        * *version_number* (``int``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['status', 'status_details', 'version_number']

    def __init__(self, *args, **kwargs):
        r"""Initializes a JobExecutionState instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *status* (``str``)
            * *status_details* (``typing.Dict[str, str]``)
            * *version_number* (``int``)
        """

        self.status = kwargs.get('status')
        self.status_details = kwargs.get('status_details')
        self.version_number = kwargs.get('version_number')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['status', 'status_details', 'version_number'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> JobExecutionState
        new = cls()
        val = payload.get('status')
        if val is not None:
            new.status = val
        val = payload.get('statusDetails')
        if val is not None:
            new.status_details = val
        val = payload.get('versionNumber')
        if val is not None:
            new.version_number = val
        return new

class JobExecutionSummary(awsiot.ModeledClass):
    r"""
    Attributes:
        * *execution_number* (``int``)
        * *job_id* (``str``)
        * *last_updated_at* (``datetime.datetime``)
        * *queued_at* (``datetime.datetime``)
        * *started_at* (``datetime.datetime``)
        * *version_number* (``int``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['execution_number', 'job_id', 'last_updated_at', 'queued_at', 'started_at', 'version_number']

    def __init__(self, *args, **kwargs):
        r"""Initializes a JobExecutionSummary instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *execution_number* (``int``)
            * *job_id* (``str``)
            * *last_updated_at* (``datetime.datetime``)
            * *queued_at* (``datetime.datetime``)
            * *started_at* (``datetime.datetime``)
            * *version_number* (``int``)
        """

        self.execution_number = kwargs.get('execution_number')
        self.job_id = kwargs.get('job_id')
        self.last_updated_at = kwargs.get('last_updated_at')
        self.queued_at = kwargs.get('queued_at')
        self.started_at = kwargs.get('started_at')
        self.version_number = kwargs.get('version_number')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['execution_number', 'job_id', 'last_updated_at', 'queued_at', 'started_at', 'version_number'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> JobExecutionSummary
        new = cls()
        val = payload.get('executionNumber')
        if val is not None:
            new.execution_number = val
        val = payload.get('jobId')
        if val is not None:
            new.job_id = val
        val = payload.get('lastUpdatedAt')
        if val is not None:
            new.last_updated_at = datetime.datetime.fromtimestamp(val)
        val = payload.get('queuedAt')
        if val is not None:
            new.queued_at = datetime.datetime.fromtimestamp(val)
        val = payload.get('startedAt')
        if val is not None:
            new.started_at = datetime.datetime.fromtimestamp(val)
        val = payload.get('versionNumber')
        if val is not None:
            new.version_number = val
        return new

class JobExecutionsChangedEvent(awsiot.ModeledClass):
    r"""
    Attributes:
        * *jobs* (``typing.Dict[str, typing.List[JobExecutionSummary]]``)
        * *timestamp* (``datetime.datetime``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['jobs', 'timestamp']

    def __init__(self, *args, **kwargs):
        r"""Initializes a JobExecutionsChangedEvent instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *jobs* (``typing.Dict[str, typing.List[JobExecutionSummary]]``)
            * *timestamp* (``datetime.datetime``)
        """

        self.jobs = kwargs.get('jobs')
        self.timestamp = kwargs.get('timestamp')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['jobs', 'timestamp'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> JobExecutionsChangedEvent
        new = cls()
        val = payload.get('jobs')
        if val is not None:
            new.jobs = {k: [JobExecutionSummary.from_payload(i) for i in v] for k,v in val.items()}
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        return new

class JobExecutionsChangedSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a JobExecutionsChangedSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *thing_name* (``str``)
        """

        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['thing_name'], args):
            setattr(self, key, val)

class JobStatus:
    CANCELED = 'CANCELED'
    FAILED = 'FAILED'
    QUEUED = 'QUEUED'
    IN_PROGRESS = 'IN_PROGRESS'
    SUCCEEDED = 'SUCCEEDED'
    TIMED_OUT = 'TIMED_OUT'
    REJECTED = 'REJECTED'
    REMOVED = 'REMOVED'

class NextJobExecutionChangedEvent(awsiot.ModeledClass):
    r"""
    Attributes:
        * *execution* (``JobExecutionData``)
        * *timestamp* (``datetime.datetime``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['execution', 'timestamp']

    def __init__(self, *args, **kwargs):
        r"""Initializes a NextJobExecutionChangedEvent instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *execution* (``JobExecutionData``)
            * *timestamp* (``datetime.datetime``)
        """

        self.execution = kwargs.get('execution')
        self.timestamp = kwargs.get('timestamp')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['execution', 'timestamp'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> NextJobExecutionChangedEvent
        new = cls()
        val = payload.get('execution')
        if val is not None:
            new.execution = JobExecutionData.from_payload(val)
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        return new

class NextJobExecutionChangedSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a NextJobExecutionChangedSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *thing_name* (``str``)
        """

        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['thing_name'], args):
            setattr(self, key, val)

class RejectedError(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *code* (``str``)
        * *execution_state* (``JobExecutionState``)
        * *message* (``str``)
        * *timestamp* (``datetime.datetime``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'code', 'execution_state', 'message', 'timestamp']

    def __init__(self, *args, **kwargs):
        r"""Initializes a RejectedError instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *code* (``str``)
            * *execution_state* (``JobExecutionState``)
            * *message* (``str``)
            * *timestamp* (``datetime.datetime``)
        """

        self.client_token = kwargs.get('client_token')
        self.code = kwargs.get('code')
        self.execution_state = kwargs.get('execution_state')
        self.message = kwargs.get('message')
        self.timestamp = kwargs.get('timestamp')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'code', 'execution_state', 'message', 'timestamp'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> RejectedError
        new = cls()
        val = payload.get('clientToken')
        if val is not None:
            new.client_token = val
        val = payload.get('code')
        if val is not None:
            new.code = val
        val = payload.get('executionState')
        if val is not None:
            new.execution_state = JobExecutionState.from_payload(val)
        val = payload.get('message')
        if val is not None:
            new.message = val
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        return new

class RejectedErrorCode:
    INTERNAL_ERROR = 'InternalError'
    INVALID_JSON = 'InvalidJson'
    INVALID_REQUEST = 'InvalidRequest'
    INVALID_STATE_TRANSITION = 'InvalidStateTransition'
    RESOURCE_NOT_FOUND = 'ResourceNotFound'
    VERSION_MISMATCH = 'VersionMismatch'
    INVALID_TOPIC = 'InvalidTopic'
    REQUEST_THROTTLED = 'RequestThrottled'
    TERMINAL_STATE_REACHED = 'TerminalStateReached'

class StartNextJobExecutionResponse(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *execution* (``JobExecutionData``)
        * *timestamp* (``datetime.datetime``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'execution', 'timestamp']

    def __init__(self, *args, **kwargs):
        r"""Initializes a StartNextJobExecutionResponse instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *execution* (``JobExecutionData``)
            * *timestamp* (``datetime.datetime``)
        """

        self.client_token = kwargs.get('client_token')
        self.execution = kwargs.get('execution')
        self.timestamp = kwargs.get('timestamp')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'execution', 'timestamp'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> StartNextJobExecutionResponse
        new = cls()
        val = payload.get('clientToken')
        if val is not None:
            new.client_token = val
        val = payload.get('execution')
        if val is not None:
            new.execution = JobExecutionData.from_payload(val)
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        return new

class StartNextPendingJobExecutionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *status_details* (``typing.Dict[str, str]``)
        * *step_timeout_in_minutes* (``int``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'status_details', 'step_timeout_in_minutes', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a StartNextPendingJobExecutionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *status_details* (``typing.Dict[str, str]``)
            * *step_timeout_in_minutes* (``int``)
            * *thing_name* (``str``)
        """

        self.client_token = kwargs.get('client_token')
        self.status_details = kwargs.get('status_details')
        self.step_timeout_in_minutes = kwargs.get('step_timeout_in_minutes')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'status_details', 'step_timeout_in_minutes', 'thing_name'], args):
            setattr(self, key, val)

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.client_token is not None:
            payload['clientToken'] = self.client_token
        if self.status_details is not None:
            payload['statusDetails'] = self.status_details
        if self.step_timeout_in_minutes is not None:
            payload['stepTimeoutInMinutes'] = self.step_timeout_in_minutes
        return payload

class StartNextPendingJobExecutionSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a StartNextPendingJobExecutionSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *thing_name* (``str``)
        """

        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['thing_name'], args):
            setattr(self, key, val)

class UpdateJobExecutionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *execution_number* (``int``)
        * *expected_version* (``int``)
        * *include_job_document* (``bool``)
        * *include_job_execution_state* (``bool``)
        * *job_id* (``str``)
        * *status* (``str``)
        * *status_details* (``typing.Dict[str, str]``)
        * *step_timeout_in_minutes* (``int``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'execution_number', 'expected_version', 'include_job_document', 'include_job_execution_state', 'job_id', 'status', 'status_details', 'step_timeout_in_minutes', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a UpdateJobExecutionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *execution_number* (``int``)
            * *expected_version* (``int``)
            * *include_job_document* (``bool``)
            * *include_job_execution_state* (``bool``)
            * *job_id* (``str``)
            * *status* (``str``)
            * *status_details* (``typing.Dict[str, str]``)
            * *step_timeout_in_minutes* (``int``)
            * *thing_name* (``str``)
        """

        self.client_token = kwargs.get('client_token')
        self.execution_number = kwargs.get('execution_number')
        self.expected_version = kwargs.get('expected_version')
        self.include_job_document = kwargs.get('include_job_document')
        self.include_job_execution_state = kwargs.get('include_job_execution_state')
        self.job_id = kwargs.get('job_id')
        self.status = kwargs.get('status')
        self.status_details = kwargs.get('status_details')
        self.step_timeout_in_minutes = kwargs.get('step_timeout_in_minutes')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'execution_number', 'expected_version', 'include_job_document', 'include_job_execution_state', 'job_id', 'status', 'status_details', 'step_timeout_in_minutes', 'thing_name'], args):
            setattr(self, key, val)

    def to_payload(self):
        # type: () -> typing.Dict[str, typing.Any]
        payload = {} # type: typing.Dict[str, typing.Any]
        if self.client_token is not None:
            payload['clientToken'] = self.client_token
        if self.execution_number is not None:
            payload['executionNumber'] = self.execution_number
        if self.expected_version is not None:
            payload['expectedVersion'] = self.expected_version
        if self.include_job_document is not None:
            payload['includeJobDocument'] = self.include_job_document
        if self.include_job_execution_state is not None:
            payload['includeJobExecutionState'] = self.include_job_execution_state
        if self.status is not None:
            payload['status'] = self.status
        if self.status_details is not None:
            payload['statusDetails'] = self.status_details
        if self.step_timeout_in_minutes is not None:
            payload['stepTimeoutInMinutes'] = self.step_timeout_in_minutes
        return payload

class UpdateJobExecutionResponse(awsiot.ModeledClass):
    r"""
    Attributes:
        * *client_token* (``str``)
        * *execution_state* (``JobExecutionState``)
        * *job_document* (``typing.Dict[str, typing.Any]``)
        * *timestamp* (``datetime.datetime``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['client_token', 'execution_state', 'job_document', 'timestamp']

    def __init__(self, *args, **kwargs):
        r"""Initializes a UpdateJobExecutionResponse instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *client_token* (``str``)
            * *execution_state* (``JobExecutionState``)
            * *job_document* (``typing.Dict[str, typing.Any]``)
            * *timestamp* (``datetime.datetime``)
        """

        self.client_token = kwargs.get('client_token')
        self.execution_state = kwargs.get('execution_state')
        self.job_document = kwargs.get('job_document')
        self.timestamp = kwargs.get('timestamp')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['client_token', 'execution_state', 'job_document', 'timestamp'], args):
            setattr(self, key, val)

    @classmethod
    def from_payload(cls, payload):
        # type: (typing.Dict[str, typing.Any]) -> UpdateJobExecutionResponse
        new = cls()
        val = payload.get('clientToken')
        if val is not None:
            new.client_token = val
        val = payload.get('executionState')
        if val is not None:
            new.execution_state = JobExecutionState.from_payload(val)
        val = payload.get('jobDocument')
        if val is not None:
            new.job_document = val
        val = payload.get('timestamp')
        if val is not None:
            new.timestamp = datetime.datetime.fromtimestamp(val)
        return new

class UpdateJobExecutionSubscriptionRequest(awsiot.ModeledClass):
    r"""
    Attributes:
        * *job_id* (``str``)
        * *thing_name* (``str``)

    All attributes are None by default, and may be set by keyword in the constructor.
    """

    __slots__ = ['job_id', 'thing_name']

    def __init__(self, *args, **kwargs):
        r"""Initializes a UpdateJobExecutionSubscriptionRequest instance

        :param \**kwargs:
            See below

        :Keyword Arguments:
            * *job_id* (``str``)
            * *thing_name* (``str``)
        """

        self.job_id = kwargs.get('job_id')
        self.thing_name = kwargs.get('thing_name')

        # for backwards compatibility, read any arguments that used to be accepted by position
        for key, val in zip(['job_id', 'thing_name'], args):
            setattr(self, key, val)

